:- module run.

:- interface.

:- import_module io.

:- pred main(io::di, io::uo) is det.

:- implementation.

:- import_module interpreter.
:- import_module list.
:- import_module maybe.
:- import_module parse.
:- import_module parsing_utils.
:- import_module pprint.
:- import_module string.

main(!IO) :-
    io.command_line_arguments(Args, !IO),
    ( Args = [File] ->
        io.open_input(File, OpenRes, !IO),
        ( OpenRes = ok(In),
            io.read_file_as_string(In, ReadRes, !IO),
            ( ReadRes = ok(Str),
                new_src_and_ps(Str, Src, PS0),
                parse_statements(File, Src, Result, PS0, _),
                ( Result = ok(Stmts),
                    Env0 = init_env,
                    execute_block(Stmts, MaybeReturn, Env0, _, !IO),
                    ( MaybeReturn = yes(Return),
                        io.write(Return, !IO)
                    ; MaybeReturn = no
                    ),
                    io.nl(!IO)
                ; Result = error(ErrCon, Err),
                    io.write_string(io.stderr_stream, "Syntax Error: ", !IO),
                    io.write(io.stderr_stream, ErrCon, !IO),
                    io.nl(io.stderr_stream, !IO),
                    io.write(io.stderr_stream, Err, !IO),
                    io.nl(io.stderr_stream, !IO),
                    io.set_exit_status(1, !IO)
                )
            ; ReadRes = error(_, ErrCode),
                io.write_string(io.stderr_stream,
                    "Error reading file " ++ File ++ ": " ++
                    io.error_message(ErrCode), !IO),
                io.nl(io.stderr_stream, !IO),
                io.set_exit_status(1, !IO)
            )
        ; OpenRes = error(ErrCode),
            io.write_string(io.stderr_stream,
                "Error opening file " ++ File ++ ": " ++
                io.error_message(ErrCode), !IO),
            io.nl(io.stderr_stream, !IO),
            io.set_exit_status(1, !IO)
        )
    ;
        io.write_string(io.stderr_stream,
            "Error: expecting exactly one argument.\n", !IO),
        io.set_exit_status(1, !IO)
    ).
